/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_JENC_JENCIF_SERVER_H_)
#define _JENC_JENCIF_SERVER_H_

#include "jenc_JencIf_common.h"
#include "escl_rpc.h"
#include "esclrt.h"
#include "jenc_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif

#if !defined(ESRPC_FUNCTIONS_DEFINITIONS_JENCIF)
#define ESRPC_FUNCTIONS_DEFINITIONS_JENCIF


/*! @brief JencIf identifiers */
enum _JencIf_ids
{
    kJencIf_service_id = 4,
    kJencIf_escljencJpegEncInit_id = 1,
    kJencIf_escljencJpegEncRelease_id = 2,
    kJencIf_escljencJpegEncSetPictureSize_id = 3,
    kJencIf_escljencJpegEncBindNumaId_id = 4,
    kJencIf_escljencJpegEncEncode_id = 5,
    kJencIf_escljencJpegEncGetRateCtrl_id = 6,
    kJencIf_escljencJpegEncSetRateCtrl_id = 7,
    kJencIf_escljencJpegEncGetPerformance_id = 8,
};

/*! jenc interface */
//! @name JencIf
//@{
/**
 * Initializes the encoder and returns an instance of it. The configuration contains stream
 * parameters that can’t be altered during encoding.
 * \brief Create and Initialize Jpeg Encoder Intstance.
 * \param [in]    pEncCfg        Points to the escljencJpegEncCfg structure that contains the encoder’s initial
 *                               configuration parameters. Generally, the configure parameter cannot be changed
 *                               when encoding.
 * \param [inout] instAddr       Points to a space where the new jpeg encoder instance pointer will be stored.
 * \param [in]    ctx            The context is initialized in application and maybe used by EWL. For example,
 *                               when the device is open before initialize, the ctx can be use to transfer this
 *                               data. It can be NULL if application don't use it.
 * \param [out]   escljencJpegEncRet value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncInit(const escljencJpegEncCfg * pEncCfg, escljencJEncInst * pInst, uintptr_t ctx);

/**  Release Jpeg Encoder Intstance.   */
escljencJpegEncRet escljencJpegEncRelease(escljencJEncInst * pInst);

/**
 * \brief Encoding configuration.
 * \param [in]    inst           Specify the jpeg encoder instance.
 * \param [in]    pEncCfg        Points to escljencJpegEncCfg structure that contains the encoder’s configuration
 *                               parameters.
 * \param [out]   escljencJpegEncRet    value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncSetPictureSize(const escljencJEncInst * pInst, const escljencJpegEncCfg * pEncCfg);

/**
 * \brief Bind numa-id.
 * \param [in]    inst           Specify the jpeg encoder instance.
 * \param [in]    nid            numa id.
 * \param [out]   escljencJpegEncRet    value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncBindNumaId(const escljencJEncInst * pInst, uint16_t nid);

/**
 * \brief Encodes one video frame.
 * \param [in]    inst           The instance that defines the encoder in use.
 * \param [in]    pEncIn         Pointer to user provided input parameters.
 * \param [out]   pEncOut        Pointer to the JEncOut structure where the output parameters will be saved.
 * \param [out]   escljencJpegEncRet    value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncEncode(const escljencJEncInst * pInst, const escljencJpegEncIn * pEncIn, escljencJpegEncOut * pEncOut);

/**
 * \brief Get Rate Control configuration, return current rate control parameters.
 * \param [in]    inst           the instance in use.
 * \param [out]   pRateCtrl      Place where parameters are returned
 * \param [out]   escljencJpegEncRet    value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncGetRateCtrl(const escljencJEncInst * pInst, escljencJpegEncRateCtrl * pRateCtrl);

/**
 * \brief Set Rate Control configuration parameters
 * \param [in]    inst           the instance in use.
 * \param [out]   pRateCtrl      Place where parameters are returned
 * \param [out]   escljencJpegEncRet    value: JPEGENC_OK initialize successfully; others see escljencJpegEncRet.
 */
escljencJpegEncRet escljencJpegEncSetRateCtrl(const escljencJEncInst * pInst, const escljencJpegEncRateCtrl * pRateCtrl);

/**
 * \brief Encoder API for get jpeg encoding performance
 * \param [in]    inst           Pointer to the encoder instance to be freed.
 * \param [out]   u32            value: The number of clock cycles used to encode the previous frame.
 */
uint32_t escljencJpegEncGetPerformance(const escljencJEncInst * pInst);
//@}


#endif // ESRPC_FUNCTIONS_DEFINITIONS_JENCIF

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Initialize rpc service for JencIf.
 */
void init_JencIf(void);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Handle rpc request for JencIf.
 */
void handle_JencIf(rpc_s_ctx_t rpc, uint32_t stream_id, void *request, uint32_t request_size);

#if defined(__cplusplus)
}
#endif

#endif // _JENC_JENCIF_SERVER_H_
