// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ESCL_COMMON_LOG_H__
#define __ESCL_COMMON_LOG_H__

#include "escl_type.h"
#include "elog_file.h"
#include "elog.h"

typedef struct {
    char* filename;
    escl_u8 level;
    escl_u8 file_enable;
} ESCLLogConfig;

#if defined(__cplusplus)
extern "C" {
#endif

void escl_log_file_init(ESCLLogConfig esclLogCfg);
void escl_log_init(ESCLLogConfig esclLogCfg);
void escl_log_file_deinit(ESCLLogConfig esclLogCfg);
void escl_log_deinit(ESCLLogConfig esclLogCfg);

/**
 * @brief Log api.
 * Style 1: Logging macro using predefined LOG_TAG
 * (LOG_TAG must be #defined at file scope before use)
 * @example
 * #define LOG_TAG "main"
 * #include "escl_log.h"
 *
 * @param ... printf-style format string and arguments
 */
#define escl_log_a(...) log_a(__VA_ARGS__)
#define escl_log_e(...) log_e(__VA_ARGS__)
#define escl_log_w(...) log_w(__VA_ARGS__)
#define escl_log_i(...) log_i(__VA_ARGS__)
#define escl_log_d(...) log_d(__VA_ARGS__)
#define escl_log_v(...) log_v(__VA_ARGS__)

/**
 * @brief Log api.
 * Style 2: Logging macro that requires explicit tag parameter for each call
 * @param tag The module tag/identifier (string literal)
 * @param ... printf-style format string and arguments.
 */
#define escl_log_assert(tag, ...) elog_assert(tag, __VA_ARGS__)
#define escl_log_error(tag, ...) elog_error(tag, __VA_ARGS__)
#define escl_log_warn(tag, ...) elog_warn(tag, __VA_ARGS__)
#define escl_log_info(tag, ...) elog_info(tag, __VA_ARGS__)
#define escl_log_debug(tag, ...) elog_debug(tag, __VA_ARGS__)
#define escl_log_verbose(tag, ...) elog_verbose(tag, __VA_ARGS__)

/**
 * @brief Dump the hex format data to log
 *
 * @param tag The module tag/identifier (string literal)
 * @param width hex number for every line, such as: 16, 32
 * @param buf hex buffer
 * @param size buffer size
 * @param title format title
 * @param ... args
 */
#define ESCL_LOG_DISABLED (0U)
#define ESCL_LOG_ENABLED (1U)
#define ESCL_RPC_DUMP ESCL_LOG_DISABLED

#if ESCL_RPC_DUMP
#define escl_hexdump(tag, buf, size, title, ...) elog_hexdump(tag, 16, buf, size, title, ##__VA_ARGS__)
#else
#define escl_hexdump(tag, buf, size, title, ...)
#endif

ESCLLogConfig parse_log_config(int argc, char* argv[]);

void set_host_pid(pid_t pid_t);
void set_device_pid(pid_t pid_t);

#if defined(__cplusplus)
}
#endif

#endif  //__ESCL_COMMON_LOG_H__