/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#ifndef __ESCL__ESCL_RPC_H__
#define __ESCL__ESCL_RPC_H__

#include "escl_type.h"
#include <stdint.h>
#include <stddef.h>
#include <sys/types.h>
#include <stdbool.h>
#include "escl_log.h"

#if defined __cplusplus
extern "C" {
#endif

#define MAX_RPMSG_BUF_SIZE (MAX_RPC_MSG_BUF_SIZE)
#define RPMSG_ADDR_ANY 0xFFFFFFFF
#define RPMSG_MANAGEMENT_EPT_ADDR (0x16)
#define RPC_NAME_MAX 255
#define MAX_WORKERS 128
#define SHM_NAME "/tmp/my_shm"
#define SHM_SIZE 4096

struct escl_rpmsg_endpoint {
    char name[32];
    uint32_t src;
    uint32_t dst;
    int32_t rfd;
    int32_t addr;
};

typedef struct {
    pid_t host_pid;
    pid_t worker_pid;
    int active;
    uint32_t managementRfd;
} WorkerEntry;

struct ESCLSvrInitParam {
    ESCLLogConfig esclLogCfg;
    bool perfFlag;
    char esclLogFileName[128];
};

enum escl_rpc_cmd_type {
    ESCL_RPC_CMD_TYPE_SET_DEVICE = 1,
    ESCL_RPC_CMD_TYPE_CREATE_STREAM,
    ESCL_RPC_CMD_TYPE_DESTORY_STREAM,
    ESCL_RPC_CMD_TYPE_DESTORY_DEVICE,
    ESCL_RPC_CMD_TYPE_SYNC_STREAM,
    ESCL_RPC_CMD_TYPE_SYNC_DEVICE,
    ESCL_RPC_CMD_TYPE_RUN_TASK,
    ESCL_RPC_CMD_TYPE_MAX,
};

struct escl_rpc_ioctl_arg {
    int stream_id;
    int buf_size;
    void *buf;
    int timeout_ms;
};

struct escl_resp_info {
    volatile bool flag;
    volatile int ep_addr;
    volatile uint32_t rfd;
    pid_t pid;
};

typedef struct {
    bool flag;
    pthread_t pid;
} StreamPidFlag;

#define MAX_RPC_MSG_BUF_SIZE (8192)
#define RPC_MSG_RESERVED_HEADER_SIZE (64)

/**
 * rpc client opaque context
 */
typedef struct escl_rpc_clt *rpc_c_ctx_t;
/**
 * rpc server opaque context
 */
typedef struct escl_rpc_svr *rpc_s_ctx_t;

typedef void (*escl_rpc_shutdown_cb)(rpc_c_ctx_t rpc);
typedef void (*escl_rpc_server_handler)(rpc_s_ctx_t rpc, uint32_t stream_id, void *message, uint32_t message_size);
typedef void (*escl_rpmsg_ns_unbind_cb)(struct escl_rpmsg_endpoint *ept);

typedef void (*release_worker)(pid_t work_pid, uint32_t work_rfd, bool);

/**
 * @brief Server remote procedure call data
 *
 * RPMsg RPC will send request to endpoint
 */
struct escl_rpc_svr {
    /** RPMsg destination endpoint structure */
    struct escl_rpmsg_endpoint ept;

    /**/
    escl_rpmsg_ns_unbind_cb unbind_cb;

    /** Service table */
};

/**
 * @brief Client remote procedure call data
 *
 * RPMsg RPC will send request to remote and
 * wait for callback.
 */
struct escl_rpc_clt {
    /** The RPMsg Server Device */
    struct escl_rpmsg_endpoint ept;

    /** Shutdown callback function */
    escl_rpc_shutdown_cb shutdown_cb;
};

/**
 * @ingroup rpc_client
 * @internal
 *
 * @brief Initialize RPMsg remote procedure call
 *
 * This function is to initialize the remote procedure call
 * client data. RPMsg RPC will send request to remote and
 * wait for callback and load services to table
 *
 * @param[out] rpc Pointer to the client remote procedure call data
 * @param[in] dev_id ID of the device will use
 * @param[in] eptName the endpoint name
 * @param[in] src_addr The address of the endpoint
 * @param[in] dst_addr The address of the endpoint will send
 * @param[in] shutdown_cb Shutdown callback function
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_client_init(rpc_c_ctx_t *rpc, int dev_id, const char *eptName, uint32_t src_addr, uint32_t dst_addr,
                             escl_rpc_shutdown_cb shutdown_cb);

/**
 * @ingroup rpc_client
 * @internal
 *
 * @brief Request RPMsg RPC call (Asynchronous Non-blocking Interface)
 *
 * This function sends an RPC request asynchronously.
 * It immediately returns after queuing the message onto the RPMsg channel,
 * without blocking to wait for a response. The response will be retrieved
 * by escl_rpc_client_wait.
 *
 * @see escl_rpc_client_wait
 *
 * @param[in] rpc Pointer to client remoteproc procedure call data
 * @param[in] stream_idx the ID of stream which the task belong to
 * @param[in] request Pointer to request buffer
 * @param[in] request_size Length of the request data
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_client_send(rpc_c_ctx_t rpc, uint32_t stream_idx, const void *request, uint32_t request_size);

/**
 * @ingroup rpc_client
 * @internal
 *
 * @brief Wait RPMsg RPC response (Asynchronous Non-blocking Interface)
 *
 * This function wait a RPC response synchronously.
 * It will block the calling thread until a response message is received or a timeout occurs.
 *
 * NOTICE: This function only waits for responses that need to be processed by the current thread.
 *
 * @param[in] rpc Pointer to client remoteproc procedure call data
 * @param[in] stream_idx the ID of the stream to wait
 * @param[out] response Pointer to response data buffer
 * @param[inout] response_size Pointer to the length, it means the buffer size before calling, and
 *                             actually received size after calling.
 * @param[in] timeout_ms Timeout value in milliseconds, -1 indicates waiting for ever.
 *
 * @return ESCL_SUCCESS, esclTimeout or errorcode.
 */
esclRet escl_rpc_client_wait(rpc_c_ctx_t rpc, uint32_t stream_idx, void *response, uint32_t *response_size,
                             int timeout_ms);

/**
 * @ingroup rpc_client
 * @internal
 *
 * @brief Release RPMsg remote procedure call
 *
 * This function is to release remoteproc procedure call service
 *
 * @param[in] rpc Pointer to the client remote procedure call data
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_client_release(rpc_c_ctx_t rpc);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Initialize RPMsg rpc for server
 *
 * This function create endpoint and loads services into table
 *
 * @param[out] rpc Pointer to the server rpc
 * @param[in] dev_id the device id for open commuction
 * @param[in] eptName Pointer to the endpoint name
 * @param[in] src_addr The address of the endpoint
 * @param[in] dst_addr The address of the endpoint will send
 * @param[in] rpmsg_service_server_unbind Unbind function callback
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_server_init(rpc_s_ctx_t *rpc, int dev_id, const char *eptName, uint32_t src_addr, uint32_t dst_addr,
                             escl_rpmsg_ns_unbind_cb rpmsg_service_server_unbind);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Start to poll messages in a new thread.
 *
 * @param[in] rpc Pointer to the server rpc
 * @param[in] stream_id stream_ID of the task belong to
 * @param[in] handler the stream execute function pointer
 * @param[in] flag the flag pointer used to exit thread gracefully
 *
 * @return thread id
 */
pthread_t escl_rpc_server_start_poll(uint32_t rfd, uint32_t stream_id, escl_rpc_server_handler handler,
                                     release_worker releaseWorker);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Send RPC reply from server to client
 *
 * This function sends the response from the server to the client
 * after processing an RPC request. It uses the provided rpc_id to
 * correlate the reply with the original request.
 *
 * @param[in] rpc Pointer to server RPC data
 * @param[in] stream_id stream_ID of the task belong to
 * @param[in] response Pointer to response data buffer
 * @param[in] response_size Size of the response data buffer
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_server_reply(rpc_s_ctx_t rpc, uint32_t stream_id, const void *response, uint32_t response_size);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief release server resources
 *
 * This function release endpoint resources
 *
 * @param[in] rpc Pointer to server RPC data
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet escl_rpc_server_release(rpc_s_ctx_t rpc);

#if defined __cplusplus
}
#endif

#endif  //__ESCL__ESCL_RPC_H__
