// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.
#ifndef LOCKFREEVECTOR_H
#define LOCKFREEVECTOR_H

#include <vector>
#include <atomic>
#include <mutex>
#include <memory>
#include <cassert>

// 原子计数器，每个独占一个缓存行
struct alignas(64) AtomicCounter {
    std::atomic<size_t> value;

    AtomicCounter() noexcept : value(0) {}
    explicit AtomicCounter(size_t v) noexcept : value(v) {}

    // 仅支持移动语义
    AtomicCounter(AtomicCounter&& other) noexcept : value(other.value.load()) {}
    AtomicCounter& operator=(AtomicCounter&& other) noexcept
    {
        value.store(other.value.load());
        return *this;
    }

    // 禁用拷贝
    AtomicCounter(const AtomicCounter&) = delete;
    AtomicCounter& operator=(const AtomicCounter&) = delete;
};

template <typename T>
class LockFreeVector
{
 public:
    explicit LockFreeVector(size_t segment_size = 1'000'000)
        : segment_size_(segment_size), segments_(1), indices_(1), current_segment_(0)
    {
        // 预分配首个段的内存
        segments_[0].resize(segment_size_);
    }

    // 线程安全的插入操作
    void push_back(const T& val)
    {
        while (true) {
            // 获取当前活动段（内存序获取防止重排序）
            size_t seg_idx = current_segment_.load(std::memory_order_acquire);

            // 原子增加当前段的计数
            size_t idx = indices_[seg_idx].value.fetch_add(1, std::memory_order_acq_rel);

            // 当前段仍有空间
            if (idx < segment_size_) {
                // 安全写入预分配的位置
                segments_[seg_idx][idx] = val;
                return;
            } else {
                // 回滚计数器增长（需处理竞态）
                indices_[seg_idx].value.fetch_sub(1, std::memory_order_relaxed);
                ExpandSegment(seg_idx);
            }
        }
    }

    // 线程安全的元素数量获取
    size_t size() const noexcept
    {
        size_t total = 0;
        // 获取快照避免中途变化
        size_t last_seg = current_segment_.load(std::memory_order_acquire);

        // 安全计数：仅统计完全填充的段
        for (size_t i = 0; i < last_seg; ++i) {
            total += indices_[i].value.load(std::memory_order_relaxed);
        }

        // 当前段可能有部分写入
        total += indices_[last_seg].value.load(std::memory_order_acquire);
        return total;
    }

    // 清空所有数据，重置状态
    void clear() noexcept
    {
        std::lock_guard<std::mutex> lock(segment_mutex_);

        // 重置数据结构
        segments_.resize(1);
        segments_[0] = std::vector<T>();
        segments_[0].resize(segment_size_);

        indices_.resize(1);
        indices_[0].value.store(0, std::memory_order_release);

        current_segment_.store(0, std::memory_order_release);
    }

    // 获取数据的平面副本（线程安全）
    std::vector<T> flatten() const
    {
        std::vector<T> result;

        // 防止清空或扩容
        std::lock_guard<std::mutex> lock(segment_mutex_);

        // 获取当前段快照
        size_t last_seg = current_segment_.load(std::memory_order_acquire);

        // 预分配空间优化性能
        size_t total_size = 0;
        for (size_t i = 0; i <= last_seg; ++i) {
            total_size += indices_[i].value.load(std::memory_order_relaxed);
        }
        result.reserve(total_size);

        // 安全拷贝数据
        for (size_t i = 0; i <= last_seg; ++i) {
            size_t count = indices_[i].value.load(std::memory_order_acquire);
            for (size_t j = 0; j < count; ++j) {
                result.push_back(segments_[i][j]);
            }
        }

        return result;
    }

 private:
    // 扩展新段的线程安全方法
    void ExpandSegment(size_t expected_segment)
    {
        std::lock_guard<std::mutex> lock(segment_mutex_);

        // 双重检查：避免重复扩容
        if (expected_segment == current_segment_.load(std::memory_order_relaxed)) {
            size_t current_idx = indices_[expected_segment].value.load(std::memory_order_acquire);

            // 再次确认确实需要扩容
            if (current_idx >= segment_size_) {
                // 添加新段
                segments_.emplace_back();
                segments_.back().resize(segment_size_);  // 预分配内存

                indices_.emplace_back(0);

                // 更新当前段指针（内存序释放确保新段可见）
                current_segment_.store(expected_segment + 1, std::memory_order_release);
            }
        }
    }

    // 常量成员
    const size_t segment_size_;

    // 数据存储：vector-of-vectors
    std::vector<std::vector<T>> segments_;

    // 每个段的计数
    std::vector<AtomicCounter> indices_;

    // 当前活动段
    std::atomic<size_t> current_segment_;

    // 段结构修改的互斥锁
    mutable std::mutex segment_mutex_;
};

#endif  // LOCKFREEVECTOR_H