/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_VENC_VENCIF_SERVER_H_)
#define _VENC_VENCIF_SERVER_H_

#include "venc_VencIf_common.h"
#include "escl_rpc.h"
#include "esclrt.h"
#include "venc_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif

#if !defined(ESRPC_FUNCTIONS_DEFINITIONS_VENCIF)
#define ESRPC_FUNCTIONS_DEFINITIONS_VENCIF


/*! @brief VencIf identifiers */
enum _VencIf_ids
{
    kVencIf_service_id = 5,
    kVencIf_esclvencEWLInit_id = 1,
    kVencIf_esclvencEWLRelease_id = 2,
    kVencIf_esclvencEWLAttach_id = 3,
    kVencIf_esclvencEWLDetach_id = 4,
    kVencIf_esclvencEWLGetCoreNum_id = 5,
    kVencIf_esclvencEWLMallocLinear_id = 6,
    kVencIf_esclvencEWLBindNumaId_id = 7,
    kVencIf_esclvencVCEncInit_id = 8,
    kVencIf_esclvencVCEncRelease_id = 9,
    kVencIf_esclvencVCEncBindNumaId_id = 10,
    kVencIf_esclvencVCEncStrmStart_id = 11,
    kVencIf_esclvencVCEncStrmEnd_id = 12,
    kVencIf_esclvencVCEncStrmEncode_id = 13,
    kVencIf_esclvencVCEncFlush_id = 14,
    kVencIf_esclvencVCEncGetPreProcessing_id = 15,
    kVencIf_esclvencVCEncSetPreProcessing_id = 16,
    kVencIf_esclvencVCEncGetRateCtrl_id = 17,
    kVencIf_esclvencVCEncSetRateCtrl_id = 18,
    kVencIf_esclvencVCEncGetCodingCtrl_id = 19,
    kVencIf_esclvencVCEncSetCodingCtrl_id = 20,
    kVencIf_esclvencVCEncGetAlignedStride_id = 21,
    kVencIf_esclvencVCEncGetCuInfo_id = 22,
    kVencIf_esclvencVCEncGetPerformance_id = 23,
    kVencIf_esclvencVCEncGetAsicConfig_id = 24,
    kVencIf_esclvencVCEncGetRoiMapVersion_id = 25,
    kVencIf_esclvencVCEncSetSeiUserData_id = 26,
    kVencIf_esclvencVCEncGetApiVersion_id = 27,
    kVencIf_esclvencVCEncGetBuild_id = 28,
    kVencIf_esclvencVCEncLogInit_id = 29,
};

/*! venc interface */
//! @name VencIf
//@{
/**
 * \brief Initialize the EWL instance.
 * esclvencEWLInit is called when the encoder instance is initialized.
 * \param [in]    param        the configure for this instance
 * \param [out]   uintptr      value:  a wrapper instance or NULL for error.
 */
uintptr_t esclvencEWLInit(const esclvencEWLInitParam_t * param);

/**
 * \brief Release the EWL instance. It is called when the encoder instance is released
 * \param [in]    inst         Specify the encoder instance.
 * \param [out]   s32          value:  success or error.
 */
int32_t esclvencEWLRelease(uintptr_t inst);

void esclvencEWLAttach(uintptr_t ctx, int32_t slice_idx, int32_t use_vcmd);

void esclvencEWLDetach(void);

/**
 * \brief Gets the number of cores.
 * \param [in]    ctx          context to access this hardware if needed.
 * \param [out]   s32          value:  the number of cores.
 */
uint32_t esclvencEWLGetCoreNum(uintptr_t ctx);

/**
 * \brief SW/HW shared memory without Memory Synchronize.
 * \param [in]    inst         Specify the encoder instance.
 * \param [in]    size         mem size.
 * \param [in]    alignment    buf alignment.
 * \param [in]    info         memory info.
 * \param [out]   s32          value:  the number of cores.
 */
int32_t esclvencEWLMallocLinear(uintptr_t inst, uint32_t size, uint32_t alignment, esclvencEWLLinearMem_t * info);

/**
 * \brief Bind numa-id.
 * \param [in]    inst         Specify the encoder instance.
 * \param [in]    nid          numa id.
 */
void esclvencEWLBindNumaId(uintptr_t inst, uint16_t nid);

/**
 * \brief Create and Initialize Video Encoder Intstance. \n
 * Initializes the encoder and returns an instance of it. The configuration contains stream
 * parameters that can’t be altered during encoding. \n
 * \param [in]    config       Points to the VCEncConfig structure that contains the encoder’s initial configuration
 *                             parameters. Generally, the configure parameter cannot be changed when encoding.
 * \param [inout] inst         Points to a space where the new encoder instance pointer will be stored.
 * \param [in]    ctx          The context is initialized in application and maybe used by EWL. For example,
 *                             when the device is open before initialize, the ctx can be use to transfer this
 *                             data. It can be NULL if application don't use it.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncInit(const esclvencVCEncConfig * config, esclvencVEncInst * inst, uintptr_t ctx);

/**
 * \brief Releases encoder instance and all associated resource. This will free all the resources allocated
 * at the encoder initialization phase. \n
 * \param [in]    inst         The encoder instance to be released.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncRelease(esclvencVEncInst * pInst);

/**
 * \brief Video Encoder Intstance binds NumaId. \n
 * \param [in]    inst         The encoder instance to be released.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncBindNumaId(const esclvencVEncInst * pInst, uint16_t nid);

/**
 * \brief Generates the SPS and PPS. \n
 * SPS is the first NAL unit and PPS is the second NAL unit.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pEncIn       Pointer to user provided input parameters.
 * \param [out]   pEncOut      pEncOut->NaluSizeBuf indicates the size of NAL units.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncStrmStart(const esclvencVEncInst * pInst, const esclvencVCEncIn * pEncIn, esclvencVCEncOut * pEncOut);

/**
 * \brief Ends a stream with an EOS code.
* \param [in]    inst         The instance that defines the encoder in use.
* \param [in]    pEncIn       Pointer to user provided input parameters.
* \param [out]   pEncOut      Pointer to the VCEncOut structure where the output parameters will be saved.
* \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
*/
esclvencVCEncRet esclvencVCEncStrmEnd(const esclvencVEncInst * pInst, const esclvencVCEncIn * pEncIn, esclvencVCEncOut * pEncOut);

/**
 * \brief Encodes one video frame.
 * If SEI messages are enabled the first NAL unit is a SEI message.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pEncIn       Pointer to user provided input parameters.
 * \param [in]    sliceReadyCb This parameter is not used.
 * \param [in]    pAppData     This parameter is not used.
 * \param [out]   pEncOut      Pointer to the VCEncOut structure where the output parameters will be saved.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncStrmEncode(const esclvencVEncInst * pInst, const esclvencVCEncIn * pEncIn, esclvencVCEncOut * pEncOut, uintptr_t sliceReadyCb, uintptr_t pAppData);

/**
 * \brief Flushes out remaining frames after all the frames to be encoded have been send into encoder.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pEncIn       Pointer to user provided input parameters.
 * \param [in]    sliceReadyCb This parameter is not used.
 * \param [in]    pAppData     This parameter is not used.
 * \param [out]   pEncOut      Pointer to the VCEncOut structure where the output parameters will be saved.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncFlush(const esclvencVEncInst * pInst, const esclvencVCEncIn * pEncIn, esclvencVCEncOut * pEncOut, uintptr_t sliceReadyCb, uintptr_t pAppData);

/**
 * \brief Get PreProcessing configuration before and during stream generation
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [out]   preProcCfg   Pointer to esclvencVCEncPreProcessingCfg structure where the new parameters are set.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetPreProcessing(const esclvencVEncInst * pInst, esclvencVCEncPreProcessingCfg * preProcCfg);

/**
 * \brief Set PreProcessing configuration before and during stream generation
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [out]   preProcCfg   Pointer to esclvencVCEncPreProcessingCfg structure where the new parameters are set.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncSetPreProcessing(const esclvencVEncInst * pInst, const esclvencVCEncPreProcessingCfg * preProcCfg);

/**
 * \brief Get Rate Control configuration, return current rate control parameters.
 * \param [in]    inst         the instance in use.
 * \param [out]   pRateCtrl    Place where parameters are returned
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetRateCtrl(const esclvencVEncInst * pInst, esclvencVCEncRateCtrl * pRateCtrl);

/**
 * \brief Set Rate Control configuration parameters
 * The function should be called before starting the stream.
 * If the Hypothetical Reference Decoder(HRD) is disabled, the function can also be called between frames to set new
 * rate control parameters except the ctbRc parameter. This however will reset the rate control and is recommended
 * to do so at the end of Group of Pictures (GOP). To modify ctbRc between frames, cu_qp_delta_enabled_flag in the
 * Picture Parameter Set (PPS) will be reconfigured, so the following steps should be done:
 * esclvencVCEncStrmEnd(): to end the last sequence
 * esclvencVCEncSetRateCtrl(): 	to change ctbRc
 * esclvencVCEncStrmStart():	to output a new PPS
 * Then, call
 * esclvencVCEncStrmEncode():	to encode the next frame
 * \param [in]    inst         the instance in use.
 * \param [out]   pRateCtrl    Place where parameters are returned
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncSetRateCtrl(const esclvencVEncInst * pInst, const esclvencVCEncRateCtrl * pRateCtrl);

/**
 * \brief Get Encoder configuration before stream generation.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [out]   pCodeParams   Pointer to the esclvencVCEncCodingCtrl structure where the encoder’s coding paras
                               will be saved.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetCodingCtrl(const esclvencVEncInst * pInst, esclvencVCEncCodingCtrl * pCodeParams);

/**
 * \brief Set Encoder configuration. All the parameters can be adjusted before the stream is started.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pCodeParams  Pointer to the EncCodingCtrl structure that contains the encoder’s coding parameters.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncSetCodingCtrl(const esclvencVEncInst * pInst, const esclvencVCEncCodingCtrl * pCodeParams);

/**
 * \brief Returns the stride in byte by given aligment and format.
 * \param [in]    width         Width of the input image.
 * \param [in]    format        Format of the input image.
 * \param [in]    alignment     Input alignment.
 * \param [out]   luma_stride   Format of the input image.
 * \param [out]   chroma_stride Chroma stride.
 * \param [out]   u32           value: Number of bytes per pixel.
 */
uint32_t esclvencVCEncGetAlignedStride(int32_t width, int32_t format, uintptr_t * luma_stride, uintptr_t * chroma_stride, uint32_t alignment);

/**
 * \brief Get the encoding information of a specified CU in a specified CTU, only valid for HEVC.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pCuOutData   Structure containning ctu table and cu information stream output by HW
 * \param [in]    ctuNum       Ctu number within picture
 * \param [in]    cuNum        Cu number within ctu
 * \param [out]   pEncCuInfo   Structure returns the parsed cu information
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetCuInfo(const esclvencVEncInst * pInst, const esclvencVCEncCuOutData * pCuOutData, uint32_t ctuNum, uint32_t cuNum, esclvencVCEncCuInfo * pEncCuInfo);

/**
 * \brief To get HW cycle number use to encode one frame.
 * \param [in]    inst         Pointer to the encoder instance to be freed.
 * \param [out]   u32          value: The number of clock cycles used to encode the previous frame.
 */
uint32_t esclvencVCEncGetPerformance(const esclvencVEncInst * pInst);

/**
 * \brief Get HW configuration.
 * \param [in]    codecFormat     The codec format.
 * \param [out]   phwCfg          structure containing the ASIC hardware configuration.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 *  Refer to the Hantro EWL API document for the definition of esclvencEWLHwConfig_t.
 */
uint32_t esclvencVCEncGetAsicConfig(esclvencVCEncVideoCodecFormat codecFormat, uintptr_t ctx, esclvencEWLHwConfig_t * phwCfg);

/**
 * \brief Get roi map version.
 * \param [in]    core_id      core id.
 * \param [in]    ctx          This parameter is not used.
 * \param [out]   u32          return roi map version.
 */
uint32_t esclvencVCEncGetRoiMapVersion(uint32_t core_id, uintptr_t ctx);

/**
 * \brief Insert User Data SEI in stream.
 * \param [in]    inst         The instance that defines the encoder in use.
 * \param [in]    pUserData    Pointer to a buffer containing the user data. The encoder stores a pointer to this
 *                             buffer and reads data from the buffer during encoding of the following frames.
 *                             Because the encoder reads data straight from this buffer, it must not be freed before
 *                             disabling the user data writing.
 * \param [in]    userDataSize Size of the data in the pUserData buffer in bytes. If a zero value is given, the user
                               data writing is disabled. Invalid value disables user data writing.
 *                             Valid value range: 0 or [16, 2048].
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncSetSeiUserData(const esclvencVEncInst * pInst, uintptr_t pUserData, uint32_t userDataSize);

/**
 * \brief Return the API version information
 * \param [out]   pVCEncApiVer value: return venc api version.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetApiVersion(esclvencVCEncApiVersion * pVCEncApiVer);

/**
 * \brief Get build info.
 * \param [in]    client_type     Used to select available HW engine when initializing the EWL
 * \param [out]   pVCEncBuild  value: return build info.
 * \param [out]   esclvencVCEncRet value: VCENC_OK initialize successfully; others see esclvencVCEncRet.
 */
esclvencVCEncRet esclvencVCEncGetBuild(uint32_t client_type, esclvencVCEncBuild * pVCEncBuild);

/**
 * \brief Initialize the log message system.
 * \param [in]    out_dir      to control log message output device. follow type definition of "vcenc_log_output"
 * \param [in]    out_level    to control log message output level.
 * \param [in]    trace_map    to control log message output trace information for prompt and debug.
 * \param [in]    check_map    to control log message output check information for test only.
 * \param [out]   s32          value: init log success or fail.
 */
int32_t esclvencVCEncLogInit(uint32_t out_dir, uint32_t out_level, uint32_t trace_map, uint32_t check_map);
//@}


#endif // ESRPC_FUNCTIONS_DEFINITIONS_VENCIF

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Initialize rpc service for VencIf.
 */
void init_VencIf(void);

/**
 * @ingroup rpc_server
 * @internal
 *
 * @brief Handle rpc request for VencIf.
 */
void handle_VencIf(rpc_s_ctx_t rpc, uint32_t stream_id, void *request, uint32_t request_size);

#if defined(__cplusplus)
}
#endif

#endif // _VENC_VENCIF_SERVER_H_
