// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#ifndef __ES_CLRT_INTERFACE_H__
#define __ES_CLRT_INTERFACE_H__

#include "escl_type.h"
#include "escl_platform_error.h"

typedef enum {
    ESCL_STREAM_STATUS_COMPLETE = 0,       // Stream上的所有任务已完成
    ESCL_STREAM_STATUS_NOT_READY = 1,      // Stream上至少有一个任务未完成
    ESCL_STREAM_STATUS_FAILED = 2,         // Stream执行过程发生错误导致失败
    ESCL_STREAM_STATUS_RESERVED = 0xFFFF,  // 预留
} esclrtStreamStatus;

typedef enum esclrtEventRecordedStatus {
    ESCL_EVENT_RECORDED_STATUS_NOT_READY = 0,  //Event未被记录到Stream中，或记录到Stream中的Event未被执行或执行失败
    ESCL_EVENT_RECORDED_STATUS_COMPLETE = 1,   //记录到Stream中的Event执行成功
} esclrtEventRecordedStatus;

typedef enum esclrtCallbackBlockType {
    ESCL_CALLBACK_NO_BLOCK,  //非阻塞
    ESCL_CALLBACK_BLOCK,  //阻塞stream中后续任务在device端的执行，后续任务等host端callback完成再执行
} esclrtCallbackBlockType;

/* context handle */
typedef void *esclrtContext;

/* stream handle */
typedef void *esclrtStream;

/* event handle */
typedef void *esclrtEvent;

typedef void *esclrtUserData;

/* user callback function */
typedef void (*esclrtCallback)(void *userData);

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif
/**
 * @brief initialize cl environment
 *
 * This function config global configuration using json file
 *
 * @param[in] configPath json config file path
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtInit(const char *configPath);

/**
 * @brief finalize cl environment
 *
 * This function would release some global resource
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtFinalize();

/**
 * @brief require available device count
 *
 * Require available device count
 *
 * @param[out] count available device count
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtGetDeviceCount(escl_u32 *count);

/**
 * @brief set device for using operation in current thread
 *
 * This function will create default context and stream
 * Should call esclrtResetDevice if you do not will use device resource
 * if call this function for same device , dont create new resource ,rather than add refcnt
 *
 * @param[in] deviceId Specific device id ,value scope: [0 - (deviceCount-1)]
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSetDevice(escl_s32 deviceId);

/**
 * @brief wait for all task to finish
 *
 * This function wait for device of current context
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSynchronizeDevice();

/**
 * @brief Get device of current context
 *
 * This function will require for device of current context
 *
 * @param[out] deviceId device id
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtGetDevice(escl_s32 *deviceId);

/**
 * @brief Reset device resource
 *
 * This function will decrease refcnt. It will release default context and stream if refcnt equal 0
 *
 * @param[in] deviceId Specific device id
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtResetDevice(escl_s32 deviceId);

/**
 * @brief Create context for specific device
 *
 * This function will create context and default stream
 * Created context will set to current thread and could use for multi thread.
 *
 * @param[in] deviceId Specific device id
 * @param[out] context created context
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtCreateContext(escl_s32 deviceId, esclrtContext *context);

/**
 * @brief Destroy specific context
 *
 * This function will destroy context until all task finished
 *
 * @param[in] context created context
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtDestroyContext(esclrtContext context);

/**
 * @brief set context to current thread
 *
 * Set context to current thread
 *
 * @param[in] context specific context
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSetCurrentContext(esclrtContext context);

/**
 * @brief Get context from current thread
 *
 * Get context from current thread
 *
 * @param[out] context specific context
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtGetCurrentContext(esclrtContext *context);

/**
 * @brief Create a stream
 *
 * Context will create default stream when context created
 * It better that explicitly create stream in complex scenario
 *
 * @param[out] stream created stream
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtCreateStream(esclrtStream *stream);

/**
 * @brief Create a evnet.
 *
 * @param[out] event created event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtCreateEvent(esclrtEvent *event);

/**
 * @brief Record an Event in the Stream.
 *
 * @param[in] event Specific event
 * @param[in] stream Specific stream
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtRecordEvent(esclrtEvent event, esclrtStream stream);

/**
 * @brief Wait an Event in the Stream.
 *
 * @param[in] stream Specific stream
 * @param[in] event Specific event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtStreamWaitEvent(esclrtStream stream, esclrtEvent event);

/**
 * @brief Wait an Event in default Stream.
 *
 * This function will wait until the event finished.
 *
 * @param[in] event Specific event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSynchronizeEvent(esclrtEvent event);

/**
 * @brief Get the execution end timestamp of an Event (represents time since AI processor system boot)
 *
 * @param[in] event Specific event
 * @param[out] timestamp The execution end timestamp of the Event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtEventGetTimestamp(esclrtEvent event, escl_u64 * timestamp);

/**
 * @brief Calculate the elapsed time between two Events (in milliseconds).
 *
 * @param[out] ms Elapsed time between start and end Events
 * @param[in] startEvent Specific start event
 * @param[in] endEvent Specific end event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtEventElapsedTime(float * ms, esclrtEvent startEvent, esclrtEvent endEvent);

/**
 * @brief Add a callback task asynchronously on the specified stream.
 *
 * @param[in] callback The callback function
 * @param[in] userData Pointer to user-defined data that will be passed to the callback function.
 * @param[in] blockType Blocking type:ESCL_CALLBACK_NO_BLOCK; ESCL_CALLBACK_BLOCK.
 * @param[in] stream The stream where callback will execute
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtLaunchCallback(esclrtCallback callback, esclrtUserData userData, esclrtCallbackBlockType blockType, esclrtStream stream);

/**
 * @brief Bind a thread to handle callback functions for a stream.
 *
 * @param[in] threadId Thread ID of the host callback thread to bind
 * @param[in] stream Target stream for callback handling.
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSubscribeReport(escl_u64 threadId, esclrtStream stream);

/**
 * @brief Waits for the execution of callback tasks dispatched by the aclrtLaunchCallback interface.
 *
 * This interface is called in a loop within the callback thread.
 *
 * @param[in] timeout_ms Wait timeout in milliseconds
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtProcessReport(escl_u32 timeout_ms);

/**
 * @brief UnBind the thread to handle callback functions for a stream.
 *
 * @param[in] threadId Thread ID of the host callback thread binded
 * @param[in] stream Target stream for callback handling.
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtUnSubscribeReport(escl_u64 threadId, esclrtStream stream);

/**
 * @brief Destroy a specific event.
 *
 * This function will wait until the event finished.
 *
 * @param[in] event specific event
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtDestroyEvent(esclrtEvent event);

/**
 * @brief Destroy a specific stream
 *
 * This function will wait until all task finished
 *
 * @param[in] stream specific stream
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtDestoryStream(esclrtStream stream);

/**
 * @brief wait until all task finished for specific stream
 *
 * @param[in] stream specific stream
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtSynchronizeStream(esclrtStream stream);

/**
 * @brief escl platform performance api
 *
 * @return ESCL_SUCCESS or errorcode.
 */
esclRet esclrtPerformanceTest();
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
#endif
