// Copyright © 2025 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#if !defined(_esclrt_memory_h_)
#define _esclrt_memory_h_
#include <stdint.h>
#include <stdlib.h>

#include "memory_type.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef uint64_t esclrtDrvMemHandle;
typedef ESCLMemCacheMode esclrtMemCacheMode;

typedef struct esclrtPhysicalMemProp {
    ESCLMemCacheMode cacheMode;
    int8_t strMmb[32];  /* name for this buffer */
    int8_t strZone[32]; /* location */
} esclrtPhysicalMemProp;

typedef ESCLMemInfo esclrtMemInfo;

/**
 * \brief Malloc size of memory from device. The access to the memory is uncached \n
 * \param [in]   size         Request memory size, the actual allocation size will be alignup to 4KB
 * \param [out]  devPtr       Point to the virtual address.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMalloc(void **devPtr, uint64_t size);

/**
 * \brief Free the memory that was allocated by esclrtMalloc previously. \n
 * \param [in]   devPtr       Point to the virtual address.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtFree(void *devPtr);

/**
 * \brief Alloc the IOVA for the device, specified by enVbUid \n
 * \param [in]   devPtr             virtual address allocated by esclrtMalloc.
 * \param [in]   enUid              Hardware module ID on the device.
 * \param [out]  iova               Value of the iova.
 * \param [out]  esclRet          value: success or error
 */
esclRet esclrtMemAllocIOVA(uint64_t *iova, void *devPtr, ESCLMemHwUid enUid);

/**
 * \brief Free the IOVA that was allocated by esclrtMemAllocIOVA previously for the specified the device \n
 * \param [in]  iova                Value of the iova allocated by esclrtMemAllocIOVA.
 * \param [in]   enUid              Hardware module ID on the device.
 * \param [out]  esclRet          value: success or error
 */
esclRet esclrtMemFreeIOVA(uint64_t iova, ESCLMemHwUid enUid);

/**
 * \brief Malloc size of memory from device. The access to the memory is cached. Use esclrtMemFlush to flush cache \n
 * \param [in]   size         Request memory size, the actual allocation size will be alignup to 4KB
 * \param [out]  devPtr       Point to the virtual address.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMallocCached(void **devPtr, uint64_t size);

/**
 * \brief Flush the memory of the device. \n
 * \param [out]  devPtr       Point to the virtual address that was allocated by esclrtMallocCached.
 * \param [in]   size         Size of the memory
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMemFlush(void *devPtr, uint64_t size);

/**
 * \brief Malloc size of memory from host. \n
 * \param [in]   size         Request memory size
 * \param [out]  hostPtr       Point to the virtual address.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMallocHost(void **hostPtr, size_t size);

/**
 * \brief Free the host memory that was allocated by esclrtMallocHost previously. \n
 * \param [in]   devPtr       Point to the virtual address.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtFreeHost(void *hostPtr);

/**
 * \brief Initialize the memory with the given value. \n
 * The memory can be either from device or host, it is determined by the virtual range.
 * \param [in]   devPtr       Point to the virtual address.
 * \param [in]   value        Initialization value.
 * \param [in]   len          Number of bytes to be initialized.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMemset(void *devPtr, int32_t value, size_t len);

/**
 * \brief Copy size of the src memory to the dst memory. \n
 * The dst and src memory can be from either host or device, it is determined by the virtual range.
 * \param [in]   dst          dest virtual address.
 * \param [in]   src          source virtual address.
 * \param [in]   len          Number of bytes to be copied.
 * be different.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMemcpy(void *dst, const void *src, size_t len);

/**
 * \brief Alloc size of physical memory from device. \n
 * \param [in]   size         Request memory size, the actual allocation size will be alignup to 4KB
 * \param [in]   prop         Property of the memory. Notes: the zone(location) of the memory can only be
 * mmz_nid_0_part_0
 * \param [out]  handle       The handle that represent this physical memory identically.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMallocPhysical(esclrtDrvMemHandle *handle, size_t size, const esclrtPhysicalMemProp *prop);

/**
 * \brief Free the physical memory that was allocated by esclrtMallocPhysical. \n
 * \param [in]   handle       The handle that represent this physical memory identically.
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtFreePhysical(esclrtDrvMemHandle handle);

/**
 * \brief map the physical memory to the virtual address in user space on the device side. \n
 * \param [in]   size         Size of the physical memory allocated by esclrtMallocPhysical.
 * \param [in]   handle       The handle that represent this physical memory identically.
 * \param [in]   cacheMode    Access to the virtual address is cached or uncached. If it's cached, use esclrtMemFlush to
 * flush cache.
 * \param [out]  virPtr       Virtual address
 * \param [out]  esclRet    value: success or error
 */
esclRet esclrtMapMem(void **virPtr, size_t size, esclrtDrvMemHandle handle, esclrtMemCacheMode cacheMode);

/**
 * \brief unmap virtuall address that was mapped by esclrtMapMem. \n
 * \param [in]  virPtr       Virtual address
 * \param [out]  esclRet   value: success or error
 */
esclRet esclrtUnmapMem(void *virPtr);

/**
 * \brief Get the device memory information. \n
 * \param [out]  memInfo     Device memory information
 * \param [out]  esclRet   value: success or error
 */
esclRet esclrtGetMemInfo(esclrtMemInfo *memInfo);

/**
 * \brief Get the file descreptor from physcal handle. \n
 * \param [in]   handle   physcal memory handle
 * \param [out]  fd       file descreptor
 * \param [out]  esclRet   value: success or error
 */
int esclrtHandle2InternalFd(esclrtDrvMemHandle handle);

#if defined(__cplusplus)
}
#endif

#endif  // _esclrt_memory_h_
