/*
 * Copyright (c) 2019-2025 ESWIN Computing
 *
 * All rights reserved.
 *
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain all
 * intellectual property and proprietary rights in and to this software. Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 */
#if !defined(_VENC_VENCIF_COMMON_H_)
#define _VENC_VENCIF_COMMON_H_
#include "venc_type.h"

#if defined(__cplusplus)
extern "C"
{
#endif
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if !defined(ESRPC_TYPE_DEFINITIONS_VENC)
#define ESRPC_TYPE_DEFINITIONS_VENC

// Enumerators data types declarations
typedef enum esclvencVCEncRet
{
    /** Success */
    ESCL_VCENC_OK = 0,
    /** A frame encoding was finished. */
    ESCL_VCENC_FRAME_READY = 1,
    /** A frame inside the encoder’s internal queue but not encoded yet, will be encoded and output in the following
       call to the encoder API */
    ESCL_VCENC_FRAME_ENQUEUE = 2,
    /** Encoder error. */
    ESCL_VCENC_ERROR = -1,
    /** Error, a pointer argument had an invalid NULL value. */
    ESCL_VCENC_NULL_ARGUMENT = -2,
    /** Error, one of the arguments was invalid. None of the arguments was set. */
    ESCL_VCENC_INVALID_ARGUMENT = -3,
    /** Error, the encoder was not able to allocate memory. */
    ESCL_VCENC_MEMORY_ERROR = -4,
    /** Error, the encoder’s system interface failed to initialize. */
    ESCL_VCENC_EWL_ERROR = -5,
    /** Error, the system interface failed to allocate memory. */
    ESCL_VCENC_EWL_MEMORY_ERROR = -6,
    /** Error, the stream was started with Hypothetical Reference Decoder (HRD) enabled and consequently, the rate
       control parameters cannot be altered. */
    ESCL_VCENC_INVALID_STATUS = -7,
    /** Error, the output buffer’s size was too small to fit the generated stream. Allocate a bigger buffer and try
       again. */
    ESCL_VCENC_OUTPUT_BUFFER_OVERFLOW = -8,
    /** Error, this can be caused by invalid bus addresses that push the encoder to access an invalid memory area. New
       frame encoding must be started. */
    ESCL_VCENC_HW_BUS_ERROR = -9,
    /** Error in the hardware data. */
    ESCL_VCENC_HW_DATA_ERROR = -10,
    /** Error, the wait for a hardware finish has timed out. The current frame is lost. New frame encoding must be
       started. */
    ESCL_VCENC_HW_TIMEOUT = -11,
    /** Error, the hardware could not be reserved for exclusive access. */
    ESCL_VCENC_HW_RESERVED = -12,
    /** Error, a fatal system error occurred. The encoding can’t continue. The encoder instance must be released.*/
    ESCL_VCENC_SYSTEM_ERROR = -13,
    /** Error, the encoder instance is invalid or corrupted. */
    ESCL_VCENC_INSTANCE_ERROR = -14,
    /** Hypothetical Reference Decoder error. */
    ESCL_VCENC_HRD_ERROR = -15,
    /** Error, the hardware was reset by external means. The whole frame is lost. */
    ESCL_VCENC_HW_RESET = -16
} esclvencVCEncRet;

typedef enum esclvencVCEncStreamType
{
    ESCL_VCENC_BYTE_STREAM = 0,        /**< NAL unit starts with hex bytes '00 00 00 01' */
    ESCL_VCENC_NAL_UNIT_STREAM = 1     /**< Plain NAL units without startcode */
} esclvencVCEncStreamType;

typedef enum esclvencVCEncProfile
{
    ESCL_VCENC_HEVC_MAIN_PROFILE = 0,
    ESCL_VCENC_HEVC_MAIN_STILL_PICTURE_PROFILE = 1,
    ESCL_VCENC_HEVC_MAIN_10_PROFILE = 2,
    ESCL_VCENC_H264_BASE_PROFILE = 9,
    ESCL_VCENC_H264_MAIN_PROFILE = 10,
    ESCL_VCENC_H264_HIGH_PROFILE = 11,
    ESCL_VCENC_H264_HIGH_10_PROFILE = 12
} esclvencVCEncProfile;

typedef enum esclvencVCEncLevel
{
    ESCL_VCENC_HEVC_LEVEL_1 = 30,
    ESCL_VCENC_HEVC_LEVEL_2 = 60,
    ESCL_VCENC_HEVC_LEVEL_2_1 = 63,
    ESCL_VCENC_HEVC_LEVEL_3 = 90,
    ESCL_VCENC_HEVC_LEVEL_3_1 = 93,
    ESCL_VCENC_HEVC_LEVEL_4 = 120,
    ESCL_VCENC_HEVC_LEVEL_4_1 = 123,
    ESCL_VCENC_HEVC_LEVEL_5 = 150,
    ESCL_VCENC_HEVC_LEVEL_5_1 = 153,
    ESCL_VCENC_HEVC_LEVEL_5_2 = 156,
    ESCL_VCENC_HEVC_LEVEL_6 = 180,
    ESCL_VCENC_HEVC_LEVEL_6_1 = 183,
    ESCL_VCENC_HEVC_LEVEL_6_2 = 186,
    ESCL_VCENC_H264_LEVEL_1 = 10,
    ESCL_VCENC_H264_LEVEL_1_b = 99,
    ESCL_VCENC_H264_LEVEL_1_1 = 11,
    ESCL_VCENC_H264_LEVEL_1_2 = 12,
    ESCL_VCENC_H264_LEVEL_1_3 = 13,
    ESCL_VCENC_H264_LEVEL_2 = 20,
    ESCL_VCENC_H264_LEVEL_2_1 = 21,
    ESCL_VCENC_H264_LEVEL_2_2 = 22,
    ESCL_VCENC_H264_LEVEL_3 = 30,
    ESCL_VCENC_H264_LEVEL_3_1 = 31,
    ESCL_VCENC_H264_LEVEL_3_2 = 32,
    ESCL_VCENC_H264_LEVEL_4 = 40,
    ESCL_VCENC_H264_LEVEL_4_1 = 41,
    ESCL_VCENC_H264_LEVEL_4_2 = 42,
    ESCL_VCENC_H264_LEVEL_5 = 50,
    ESCL_VCENC_H264_LEVEL_5_1 = 51,
    ESCL_VCENC_H264_LEVEL_5_2 = 52,
    ESCL_VCENC_H264_LEVEL_6 = 60,
    ESCL_VCENC_H264_LEVEL_6_1 = 61,
    ESCL_VCENC_H264_LEVEL_6_2 = 62
} esclvencVCEncLevel;

typedef enum esclvencVCEncTier
{
    ESCL_VCENC_HEVC_MAIN_TIER = 0,
    ESCL_VCENC_HEVC_HIGH_TIER = 1
} esclvencVCEncTier;

enum
{
    ESCL_MAX_VENC_STRM_BUF_NUM = 2,
    ESCL_VCENC_MAX_REF_FRAMES = 8,
    ESCL_VCENC_MAX_LT_REF_FRAMES = 8,
    ESCL_MAX_GOP_SIZE = 16,
    ESCL_MAX_GOP_PIC_CONFIG_NUM = 48,
    ESCL_MAX_GOP_CFG_SIZE = 48,
    ESCL_MAX_GOP_SPIC_CONFIG_NUM = 16,
    ESCL_MAX_CORE_NUM = 4
};

typedef enum esclvencVCEncVideoCodecFormat
{
    ESCL_VCENC_VIDEO_CODEC_HEVC = 0,
    ESCL_VCENC_VIDEO_CODEC_H264 = 1,
    ESCL_VCENC_VIDEO_CODEC_AV1 = 2
} esclvencVCEncVideoCodecFormat;

typedef enum esclvencVCEncPictureCodingType
{
    ESCL_VCENC_INTRA_FRAME = 0,
    ESCL_VCENC_PREDICTED_FRAME = 1,
    ESCL_VCENC_BIDIR_PREDICTED_FRAME = 2,
    ESCL_VCENC_NOTCODED_FRAME = 3
} esclvencVCEncPictureCodingType;

typedef enum esclvencVCEncPictureType
{
    /** YYYY... UUUU... VVVV...  */
    ESCL_VCENC_YUV420_PLANAR = 0,
    /** YYYY... UVUVUV...        */
    ESCL_VCENC_YUV420_SEMIPLANAR = 1,
    /** YYYY... VUVUVU...        */
    ESCL_VCENC_YUV420_SEMIPLANAR_VU = 2,
    /** YUYVYUYV...              */
    ESCL_VCENC_YUV422_INTERLEAVED_YUYV = 3,
    /** UYVYUYVY...              */
    ESCL_VCENC_YUV422_INTERLEAVED_UYVY = 4,
    /** 16-bit RGB 16bpp         */
    ESCL_VCENC_RGB565 = 5,
    /** 16-bit RGB 16bpp         */
    ESCL_VCENC_BGR565 = 6,
    /** 15-bit RGB 16bpp         */
    ESCL_VCENC_RGB555 = 7,
    /** 15-bit RGB 16bpp         */
    ESCL_VCENC_BGR555 = 8,
    /** 12-bit RGB 16bpp         */
    ESCL_VCENC_RGB444 = 9,
    /** 12-bit RGB 16bpp         */
    ESCL_VCENC_BGR444 = 10,
    /** 24-bit RGB 16bpp         */
    ESCL_VCENC_RGB888 = 11,
    /** 24-bit RGB 16bpp         */
    ESCL_VCENC_BGR888 = 12,
    /** 30-bit RGB 16bpp         */
    ESCL_VCENC_RGB101010 = 13,
    /** 30-bit RGB 16bpp         */
    ESCL_VCENC_BGR101010 = 14,
    /** YYYY... UUUU... VVVV...  */
    ESCL_VCENC_YUV420_PLANAR_10BIT_I010 = 15,
    /** YYYY... UVUVUV...  */
    ESCL_VCENC_YUV420_PLANAR_10BIT_P010 = 16,
    /** YYYY... UUUU... VVVV...  */
    ESCL_VCENC_YUV420_PLANAR_10BIT_PACKED_PLANAR = 17,
    /** Y0U0Y1a0a1Y2V0Y3a2a3Y4U1Y5a4a5Y6V1Y7a6a7... */
    ESCL_VCENC_YUV420_10BIT_PACKED_Y0L2 = 18,
    ESCL_VCENC_YUV420_PLANAR_8BIT_TILE_32_32 = 19,
    ESCL_VCENC_YUV420_PLANAR_8BIT_TILE_16_16_PACKED_4 = 20,
    /** YYYY... UVUVUV...        */
    ESCL_VCENC_YUV420_SEMIPLANAR_8BIT_TILE_4_4 = 21,
    /** YYYY... VUVUVU...        */
    ESCL_VCENC_YUV420_SEMIPLANAR_VU_8BIT_TILE_4_4 = 22,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_PLANAR_10BIT_P010_TILE_4_4 = 23,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_SEMIPLANAR_101010 = 24,
    /** YYYY... VUVU... */
    ESCL_VCENC_YUV420_8BIT_TILE_64_4 = 26,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_UV_8BIT_TILE_64_4 = 27,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_10BIT_TILE_32_4 = 28,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_10BIT_TILE_48_4 = 29,
    /** YYYY... VUVU... */
    ESCL_VCENC_YUV420_VU_10BIT_TILE_48_4 = 30,
    /** YYYY... VUVU... */
    ESCL_VCENC_YUV420_8BIT_TILE_128_2 = 31,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_UV_8BIT_TILE_128_2 = 32,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_10BIT_TILE_96_2 = 33,
    /** YYYY... VUVU... */
    ESCL_VCENC_YUV420_VU_10BIT_TILE_96_2 = 34,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_8BIT_TILE_8_8 = 35,
    /** YYYY... UVUV... */
    ESCL_VCENC_YUV420_10BIT_TILE_8_8 = 36,
    /** YYYY... VVVV... UUUU... */
    ESCL_VCENC_YVU420_PLANAR = 37,
    /** YYYY... UVUVUV   64x2 tile */
    ESCL_VCENC_YUV420_UV_8BIT_TILE_64_2 = 38,
    /** YYYY... UVUVUV   64x2 tile */
    ESCL_VCENC_YUV420_UV_10BIT_TILE_128_2 = 39,
    ESCL_VCENC_FORMAT_MAX = 40
} esclvencVCEncPictureType;

typedef enum esclvencVCEncPictureRotation
{
    ESCL_VCENC_ROTATE_0 = 0,
    ESCL_VCENC_ROTATE_90R = 1,
    ESCL_VCENC_ROTATE_90L = 2,
    ESCL_VCENC_ROTATE_180R = 3
} esclvencVCEncPictureRotation;

typedef enum esclvencVCEncChromaIdcType
{
    ESCL_VCENC_CHROMA_IDC_400 = 0,
    ESCL_VCENC_CHROMA_IDC_420 = 1,
    ESCL_VCENC_CHROMA_IDC_422 = 2
} esclvencVCEncChromaIdcType;

typedef enum esclvencVCENC_TuneType
{
    ESCL_VCENC_TUNE_PSNR = 0,
    ESCL_VCENC_TUNE_SSIM = 1,
    ESCL_VCENC_TUNE_VISUAL = 2,
    ESCL_VCENC_TUNE_SHARP_VISUAL = 3
} esclvencVCENC_TuneType;

typedef enum esclvencrcMode_e
{
    ESCL_VCE_RC_CVBR = 0,
    ESCL_VCE_RC_CBR = 1,
    ESCL_VCE_RC_VBR = 2,
    ESCL_VCE_RC_ABR = 3,
    ESCL_VCE_RC_CRF = 4,
    ESCL_VCE_RC_CQP = 5
} esclvencrcMode_e;

typedef enum esclvenctrue_e
{
    ESCL_ENCHW_NO = 0,
    ESCL_ENCHW_YES = 1
} esclvenctrue_e;

typedef enum esclvencCLIENT_TYPE
{
    ESCL_EWL_CLIENT_TYPE_H264_ENC = 0,    /**< H264 Encoder */
    ESCL_EWL_CLIENT_TYPE_HEVC_ENC = 1,    /**< HEVC Encoder */
    ESCL_EWL_CLIENT_TYPE_VP9_ENC = 2,    /**< VP9 Encoder */
    ESCL_EWL_CLIENT_TYPE_JPEG_ENC = 3,    /**< JPEG Encoder */
    ESCL_EWL_CLIENT_TYPE_CUTREE = 4,    /**< CU Tree Analyzer Engine */
    ESCL_EWL_CLIENT_TYPE_VIDEOSTAB = 5,    /**< Video Stabilization Engine (invalid now) */
    ESCL_EWL_CLIENT_TYPE_DEC400 = 6,    /**< DEC400 engine */
    ESCL_EWL_CLIENT_TYPE_AV1_ENC = 7,    /**< AV1 Encoder */
    ESCL_EWL_CLIENT_TYPE_L2CACHE = 8,    /**< L2CACHE engine */
    ESCL_EWL_CLIENT_TYPE_AXIFE = 9,    /**< AXIfe engine */
    ESCL_EWL_CLIENT_TYPE_APBFT = 10,   /**< APBfilter engine */
    ESCL_EWL_CLIENT_TYPE_AXIFE_1 = 11,   /**< AXIfe_1 engine */
    ESCL_EWL_CLIENT_TYPE_MEM = 12,   /**< alloc/free memory */
    ESCL_EWL_CLIENT_TYPE_MAX = 13
} esclvencCLIENT_TYPE;

typedef enum esclvenc_log_output
{
    ESCL_LOG_STDOUT = 0,
    ESCL_LOG_ONE_FILE = 1,
    ESCL_LOG_BY_THREAD = 2,
    ESCL_LOG_STDERR = 3,
    ESCL_LOG_COUNT = 4
} esclvenc_log_output;

typedef enum esclvenc_log_level
{
    ESCL_VCENC_LOG_QUIET = 0,
    ESCL_VCENC_LOG_FATAL = 1,
    ESCL_VCENC_LOG_ERROR = 2,
    ESCL_VCENC_LOG_WARN = 3,
    ESCL_VCENC_LOG_INFO = 4,
    ESCL_VCENC_LOG_DEBUG = 5,
    ESCL_VCENC_LOG_ALL = 6,
    ESCL_VCENC_LOG_COUNT = 7
} esclvenc_log_level;

// Aliases data types declarations
typedef struct esclvencEWLInitParam_t esclvencEWLInitParam_t;
typedef struct esclvencEWLLinearMem_t esclvencEWLLinearMem_t;
typedef struct esclvencVCEncApiVersion esclvencVCEncApiVersion;
typedef struct esclvencVCEncBuild esclvencVCEncBuild;
typedef struct esclvencEWLHwConfig_t esclvencEWLHwConfig_t;
typedef struct esclvencVInfoReportStat esclvencVInfoReportStat;
typedef struct esclvencVEncInst esclvencVEncInst;
typedef struct esclvencVCEncConfig esclvencVCEncConfig;
typedef struct esclvencVCEncGopPicRps esclvencVCEncGopPicRps;
typedef struct esclvencVCEncGopPicConfig esclvencVCEncGopPicConfig;
typedef struct esclvencVCEncGopPicSpecialConfig esclvencVCEncGopPicSpecialConfig;
typedef struct esclvencVCEncGopConfig esclvencVCEncGopConfig;
typedef struct esclvencExternalSEI esclvencExternalSEI;
typedef struct esclvencVCEncIn esclvencVCEncIn;
typedef struct esclvencVCEncCuOutData esclvencVCEncCuOutData;
typedef struct esclvencVCEncOut esclvencVCEncOut;
typedef struct esclvencVCEncPreProcessingCfg esclvencVCEncPreProcessingCfg;
typedef struct esclvencVCEncRateCtrl esclvencVCEncRateCtrl;
typedef struct esclvencEncPictureArea esclvencEncPictureArea;
typedef struct esclvencHdr10DisplaySei esclvencHdr10DisplaySei;
typedef struct esclvencHdr10LightLevelSei esclvencHdr10LightLevelSei;
typedef struct esclvencVuiColorDescription esclvencVuiColorDescription;
typedef struct esclvencVCEncCodingCtrl esclvencVCEncCodingCtrl;
typedef struct esclvencVCEncCuInfo esclvencVCEncCuInfo;
typedef struct esclvenclog_env_setting esclvenclog_env_setting;

// Structures/unions data types declarations
struct esclvencEWLInitParam_t
{
    uint32_t clientType;    /**< encoding format type,0: */
    uintptr_t context;   /**< initialized in application and it will be resent to application by EWL */
    uint32_t slice_idx;     /**< \brief slice node for multi-node VPU */
};

struct esclvencEWLLinearMem_t
{
    uintptr_t virtualAddress;     /**< aligned virtural address access by CPU */
    uint64_t busAddress;             /**< aligned address access by VPU */
    uint32_t size;                   /**< size in byte */
    uint32_t mem_type;               /**< Indicate the memory owners and attributes,
                                 * The bit[31:16] means access memory owners,
                                 * The bit[15:0] indicate the attributes of the memory.*/
};

struct esclvencVCEncApiVersion
{
    uint32_t major;           /**< \brief Encoder API major version */
    uint32_t minor;           /**< \brief Encoder API minor version */
    uint32_t clnum;           /**< \brief CL# */
};

struct esclvencVCEncBuild
{
    uint32_t swBuild;         /**< \brief Software build ID */
    uint32_t hwBuild;         /**< \brief Hardware build ID */
};

struct esclvencEWLHwConfig_t
{
    uint32_t roiMapVersion;
};

struct esclvencVInfoReportStat
{
    uint32_t instId;
    uint32_t running;
    uint32_t inQCnt;
    uint32_t inDqCnt;
    uint32_t outQCnt;
    uint32_t outDqCnt;
    uint32_t inWidth;
    uint32_t inHeight;
    uint32_t inBufferNum;
    uint32_t outBufferNum;
    uint64_t beginDateMs;
    uint64_t curDateMs;
    uint32_t AliveDurationMs;
    uint32_t beginMediaTs;
    uint32_t curMediaTs;
    uint32_t codec;
    uint32_t bFramesNum;
    uint32_t profile;
    uint32_t preset;
    uint32_t iFrameInterval;
    uint32_t outTotalBits;
    uint32_t timeBaseNum;
    uint32_t timeBaseDen;
    float inFrameRate;
};

struct esclvencVEncInst
{
    uintptr_t inst;
    esclvencVInfoReportStat info_report_stat;
};

struct esclvencVCEncConfig
{
    esclvencVCEncStreamType streamType;  /**< \brief Byte stream / Plain NAL units */
    esclvencVCEncProfile profile;        /**< \brief Main, Main Still or main10 */
    esclvencVCEncLevel level;            /**< \brief Level encoded in to the SPS. When set as zero, will select
                                 automatically. See \ref ss_autoLevel. */
    esclvencVCEncTier tier;              /**< \brief Main Tier or High Tier */
    uint32_t width;                     /**< \brief Encoded picture width in pixels, multiple of 2 */
    uint32_t height;                    /**< \brief Encoded picture height in pixels, multiple of 2 */
    uint32_t frameRateNum;              /**< \brief The stream time scale, [1..1048575] */
    uint32_t frameRateDenom;            /**< \brief Maximum frame rate is frameRateNum/frameRateDenom
                                 * in frames/second. The actual frame rate will be
                                 * defined by timeIncrement of encoded pictures, [1..frameRateNum] */
    uint32_t bitDepthLuma;      /**< \brief Luma sample bit depth of encoded bit stream, 8 = 8 bits, 10 = 10 bits  */
    uint32_t bitDepthChroma;    /**< \brief Chroma sample bit depth of encoded bit stream,  8 = 8 bits, 10 = 10 bits */
    esclvencVCEncVideoCodecFormat codecFormat;      /**< \brief Video Codec Format: HEVC/H264/AV1 */
    uint32_t ctbRcMode;
    uint32_t gopSize;           /**< \brief Sequence level GOP size, set gopSize=0 for adaptive GOP size. */
    uint32_t refFrameAmount;
    uint32_t strongIntraSmoothing;        /**< \brief 0 = Normal smoothing, 1 = Strong smoothing. */
    uint32_t compressor;
    uint32_t interlacedFrame;
    uint32_t enableOutputCuInfo;
    /** \brief Control CU information dumping, 1: Enable, 0: Disable. See \ref ss_outCuInfo */
    uint32_t enableOutputCtbBits;        /**< \brief 1 to enable CTB bits output. See \ref ss_outCtbBits */
    uint32_t enableSsim;                 /**< \brief Enable/Disable SSIM calculation */
    uint32_t enablePsnr;                 /**< \brief Enable/Disable PSNR calculation */
    uint32_t maxTLayers;                 /**< \brief Max number Temporal layers*/
    uint32_t rdoLevel;
    uint32_t verbose;                    /**< \brief Log printing mode */
    uint32_t exp_of_input_alignment;     /**< \brief Indicates the encoder's alignment,except input/ref frame*/
    uint32_t exp_of_ref_alignment;       /**< \brief Just affect luma reference frame config*/
    uint32_t exp_of_ref_ch_alignment;    /**< \brief Just affect chroma reference frame config*/
    uint32_t exp_of_aqinfo_alignment;    /**< \brief Just affect aqinfo output config*/
    uint32_t exteralReconAlloc;  /**< \brief 0 = recon frame is allocated by encoder itself
                              1 = recon frame is allocated by APP*/
    uint32_t P010RefEnable;      /**< \brief Enable P010 tile-raster format for reference frame buffer */
    uint32_t picOrderCntType;
    uint32_t log2MaxPicOrderCntLsb;
    uint32_t log2MaxFrameNum;
    uint32_t dumpRegister;   /**< \brief Enable dump register values for debug */
    uint32_t dumpCuInfo;     /**< \brief Enable dump the Cu Information after encoding one frame for debug */
    uint32_t dumpCtbBits;    /**< \brief Enable dump the CTB encoded bits information after encoding one frame */
    uint32_t rasterscan;     /**< \brief Enable raster recon yuv dump on FPGA & HW for debug */
    uint32_t parallelCoreNum;/**< \brief Num of Core used by one encoder instance */
    uint32_t pass;           /**< \brief Multipass coding config, currently only for internal test purpose */
    bool bPass1AdaptiveGop;
    uint8_t lookaheadDepth;
    uint32_t extDSRatio;  /**< \brief External downsample ratio for first pass. 0=1/1 (no downsample), 1=1/2 downsample */
    uint32_t inLoopDSRatio;  /**< \brief In-loop downsample ratio for first pass. 0=1/1 (no downsample), 1=1/2 downsample */
    int32_t cuInfoVersion;  /**< \brief CuInfo version; -1: decide by VC8000E and HW support; 0,1,2:different cuInfo version*/
    uint32_t slice_idx;
    /** \brief Slice node */
    /** \brief External SRAM*/
    uint32_t extSramLumHeightBwd;
    uint32_t extSramChrHeightBwd;
    uint32_t extSramLumHeightFwd;
    uint32_t extSramChrHeightFwd;
    uint64_t AXIAlignment;
    uint32_t irqTypeMask;
    uint32_t irqTypeCutreeMask;
    esclvencVCEncChromaIdcType codedChromaIdc;
    esclvencVCENC_TuneType tune;
    uint32_t writeReconToDDR;  /**< \brief HW write recon to DDR or not if it's pure I-frame encoding*/
    uint32_t TxTypeSearchEnable;  /**< \brief Av1 tx type search 1=enable 0=disable, enabled by default*/
    uint32_t av1InterFiltSwitch;  /**< \brief Av1 interp filter switchable. 1=enable 0=disable, enabled by default*/
    uint32_t fileListExist;  /**< \brief File list exist flag. 1=exist 0=not exist*/
    uint32_t burstMaxLength;  /**< \brief AXI max burst length */
    uint32_t enableTMVP;
    uint32_t bIOBufferBinding;      /**< \brief If bind input buffer and output buffer 1->bind, 0 notbind*/
    uintptr_t cb_try_new_params;  /**< \brief Callback to get new encode parameters when reEncode */
};

struct esclvencVCEncGopPicRps
{
    int32_t ref_pic;     /**< \brief  Delta_poc(Picture Order Count) of this short reference picture relative to the poc of
                        current picture or index of LTR */
    uint32_t used_by_cur; /**< \brief  whether this reference picture used by current picture (1) or not (0). */
};

struct esclvencVCEncGopPicConfig
{
    uint32_t poc;                           /**< \brief  Picture order count(POC) within a GOP, ranging from 1 to gopSize. */
    int32_t QpOffset;                      /**< \brief  Quantization Parameter(QP) offset  */
    double QpFactor;                   /**< \brief  QP Factor  */
    int32_t temporalId;                    /**< \brief  Temporal layer ID */
    esclvencVCEncPictureCodingType codingType; /**< \brief  Picture coding type  */
    uint32_t nonReference;                  /**< \brief  Frame not used for future reference */
    uint32_t numRefPics; /**< \brief  The number of reference pictures kept for this picture, the value should be within [0,
                       VCENC_MAX_REF_FRAMES] */
    esclvencVCEncGopPicRps refPics[8]; /**< \brief  Short-term ref picture sets for this pic*/
};

struct esclvencVCEncGopPicSpecialConfig
{
    uint32_t poc;                           /**< \brief Picture order count within a GOP  */
    int32_t QpOffset;                      /**< \brief  QP offset  */
    double QpFactor;                   /**< \brief  QP Factor  */
    int32_t temporalId;                    /**< \brief  Temporal layer ID */
    esclvencVCEncPictureCodingType codingType; /**< \brief  Picture coding type  */
    uint32_t nonReference;                  /**< \brief  Frame not used for future reference */
    uint32_t numRefPics; /**< \brief  The number of reference pictures kept for this picture, the value should be within [0,
                       VCENC_MAX_REF_FRAMES] */
    esclvencVCEncGopPicRps refPics[8]; /**< \brief Sshort-term ref picture sets for this pic*/
    int32_t i32Ltr;      /**< \brief Index of the long-term referencr frame,the value should be within [0,
                        VCENC_MAX_LT_REF_FRAMES]. 0 -- not LTR; 1...VCENC_MAX_LT_REF_FRAMES--- LTR's index */
    int32_t i32Offset;   /**< \brief Offset of the special pics, relative to start of ltrInterval*/
    int32_t i32Interval; /**< \brief Interval between two pictures using LTR as referencr picture or  interval between two
                        pictures coded as special frame */
    int32_t i32short_change; /**< \brief Only change short term coding parameter. 0 - not change, 1- change */
};

struct esclvencVCEncGopConfig
{
    esclvencVCEncGopPicConfig pGopPicCfg[48]; /**< \brief Pointer to an array containing all 
                                                                       used VCEncGopPicConfig */
    uint8_t size; /**< \brief The number of VCEncGopPicConfig pointed by pGopPicCfg, the value should be within [0,
                MAX_GOP_PIC_CONFIG_NUM] */
    uint8_t special_size;
    /**Number of special configuration, the value should be within [0, MAX_GOP_PIC_CONFIG_NUM] */
    uint32_t delta_poc_to_next; /**< \brief The difference between poc of next picture and current picture */
    esclvencVCEncGopPicSpecialConfig pGopPicSpecialCfg[16];
    uint8_t ltrcnt; /**< \brief Number of long-term ref pics used,the value should be within [0, VCENC_MAX_LT_REF_FRAMES] */
    uint32_t u32LTR_idx[8]; /**< \brief LTR's INDEX */
    int32_t idr_interval;                        /**< \brief idr interval */
    int32_t firstPic;                            /**< \brief First picture of input file to encode. */
    int32_t lastPic;                             /**< \brief Last picture of input file to encode. */
    int32_t outputRateNumer;                     /**< \brief Output frame rate numerator */
    int32_t outputRateDenom;                     /**< \brief Output frame rate denominator */
    int32_t inputRateNumer;                      /**< \brief Input frame rate numerator */
    int32_t inputRateDenom;                      /**< \brief Input frame rate denominator */
    int32_t gopLowdelay;
    int32_t interlacedFrame;
    /** interlace frame will be encoded. for HEVC gopSize=1 only now. See \ref ss_interlace */
    uint8_t gopCfgOffset[17]; /**< \brief Specify the index of different GOP size. */
};

struct esclvencExternalSEI
{
    uint8_t nalType;     /**< \brief External SEI NAL type,
                        * For HEVC, this should be PREFIX_SEI_NUT or SUFFIX_SEI_NUT
                        * For H264, this will always be treated as PREFIX_SEI_NUT
                        */
    uint8_t payloadType;     /**< \brief External SEI payload type, e.g. 1 for picture timing.
                        * For HEVC, please see ITU-T Rec. H.265 (11/2019), Annex D.2.1 for reference
                        * For H264, please see ITU-T Rec. H.264 (06/2019), Annex D.1.1 for reference
                        */
    uint32_t payloadDataSize;    /**< \brief External SEI payload data length */
    uintptr_t pPayloadData;   /**< \brief External SEI payload data */
};

struct esclvencVCEncIn
{
    uint64_t busLuma;         /**< \brief Bus address for input picture
                              * planar format: luminance component
                              * semiplanar format: luminance component
                              * interleaved format: whole picture
                              */
    uint64_t busChromaU;      /**< \brief Bus address for input chrominance
                              * planar format: cb component
                              * semiplanar format: both chrominance
                              * interleaved format: not used
                              */
    uint64_t busChromaV;      /**< \brief Bus address for input chrominance
                              * planar format: cr component
                              * semiplanar format: not used
                              * interleaved format: not used
                              */
    uint32_t timeIncrement;   /**< \brief The previous picture duration in units
                              * of 1/frameRateNum. 0 for the very first picture
                              * and typically equal to frameRateDenom for the rest.
                              */
    uint32_t vui_timing_info_enable; /**< \brief Write VUI timing info in SPS: 1=enable, 0=disable. */
    uintptr_t pOutBuf[2];        /**< \brief Pointer to output stream buffer */
    uint64_t busOutBuf[2];     /**< \brief Bus address of output stream buffer */
    uint32_t outBufSize[2];      /**< \brief Size of output stream buffer in bytes */
    esclvencVCEncPictureCodingType codingType;    /**< \brief Proposed picture coding type, INTRA/PREDICTED */
    int32_t poc;      /**< \brief Picture display order count */
    esclvencVCEncGopConfig gopConfig; /**< \brief GOP configuration*/
    uint32_t dec400Enable; /**< \brief 1: bypass 2: enable */
    uint32_t axiFEEnable;  /**< \brief 0: axiFE disable   1:axiFE normal mode enable   2:axiFE bypass  3.security mode*/
    int32_t picture_cnt; /**< \brief Encoded picture count */
    int32_t last_idr_picture_cnt;/**< \brief Encoded picture count of last IDR */
    int8_t i8SpecialRpsIdx;                             /**< \brief special RPS index selected by current coded picture,
                                                          *-1 - NOT, >= 0 - special RPS */
    int32_t long_term_ref_pic[8];  /**< \brief LTR's poc:-1 - not a valid poc,
                                                           *>= long-term referencr's poc */
    uint32_t bIsIDR;                                      /**< \brief Coded current picture as IDR frame ,:
                                                           *false - not coded as IDR, true-coded as IDR */
    uint32_t bIsPeriodUsingLTR;                           /**< \brief Use LTR as referencr picture periodly:
                                                           *false - not periodly used, true - periodly used */
    uint32_t bIsPeriodUpdateLTR;                           /**< \brief Update LTR periodly:false - not periodly update,
                                                            *true - periodly update */
    esclvencVCEncGopPicConfig gopCurrPicConfig;          /**< \brief Current pic running configuration */
    uint32_t bLTR_need_update[8];   /**< \brief LTR's poc should update: false -
                                                            *not need updated, true- need updated */
    uint8_t u8IdxEncodedAsLTR;                            /**< \brief current picture coded as LTR : 0 - not a LTR,
                                                            *1... VCENC_MAX_LT_REF_FRAMES -- idx of LTR */
    uintptr_t pRoiMapDelta; /**< \brief  ROI Map address for CPU access. */
    uint32_t roiMapDeltaSize;      /**< \brief ROI Map size for one frame. */
    uint64_t roiMapDeltaQpAddr;  /**< \brief ROI Map Address for encoder access. */
    int32_t gopSize;  /**< \brief current GOP size*/
    uint32_t resendVPS;      /**< \brief 0=NOT send VPS, 1= send VPS, see \ref ss_resendHeaders */
    uint32_t resendSPS;      /**< \brief 0=NOT send SPS, 1= send SPS, see \ref ss_resendHeaders */
    uint32_t resendPPS;      /**< \brief 0=NOT send PPS, 1= send PPS, see \ref ss_resendHeaders */
    uint32_t sendAUD;        /**<\brief  0=NOT send AUD, 1= send AUD */
    uint32_t bSkipFrame;   /**< \brief code all MB/CTB in current frame as SKIP Mode. 0=No, 1=Yes, see \ref ss_skipFrame */
    uint32_t externalSEICount;             /**< \brief External SEI count, see \ref ss_externSEI */
    esclvencExternalSEI pExternalSEI;        /**< \brief External SEI info array address, see \ref ss_externSEI */
};

struct esclvencVCEncCuOutData
{
    uintptr_t ctuOffset;  /**< \brief Point to a memory containing the total CU number by the end of each CTU. */
    uintptr_t cuData;     /**< \brief CU information of a picture output by HW. */
};

struct esclvencVCEncOut
{
    esclvencVCEncPictureCodingType codingType; /**< \brief Realized picture coding type, INTRA/PREDICTED/NOTCODED */
    uint32_t streamSize;                      /**< \brief Size of output stream in bytes */
    /** \brief Output buffer for NAL unit sizes pNaluSizeBuf[0] = NALU 0 size in bytes
     *pNaluSizeBuf[1] = NALU 1 size in bytes etc
     * Zero value is written after last NALU. */
    uintptr_t pNaluSizeBuf;
    uint32_t numNalus; /**< \brief Amount of NAL units */
    /** \brief 32 bit flag used for av1/vp9 codec IVF timestamp,
      each bit indicates whether a frame
      (obu frame for av1) in output buffer
      is a not-show frame. 1: not show frame; 0: show frame.
      Bit 0 of av1Vp9FrameNotShow: av1/vp9 first output frame
      (obu frame for av1) is not-show frame; Etc */
    esclvencVCEncCuOutData cuOutData; /**< \brief Pointer for cu information output by HW */
    int32_t picture_cnt; /**< \brief Encoded picture count */
    int32_t poc; /**< \brief Picture display order count */
};

struct esclvencVCEncPreProcessingCfg
{
    uint32_t origWidth;                   /**< \brief Input camera picture width */
    uint32_t origHeight;                  /**< \brief Input camera picture height*/
    uint32_t xOffset;                     /**< \brief Horizontal offset          */
    uint32_t yOffset;                     /**< \brief Vertical offset            */
    esclvencVCEncPictureType inputType;    /**< \brief Input picture color format */
    esclvencVCEncPictureRotation rotation; /**< \brief Input picture rotation     */
    int32_t constChromaEn;               /**< \brief Constant chroma control */
    uint32_t constCb;
    uint32_t constCr;
    uint32_t input_alignment;             /**< \brief Just affect prp config of input frame*/
    uint32_t inputLumaStride;
    uint32_t inputChromaStride;
};

struct esclvencVCEncRateCtrl
{
    uint32_t pictureRc;                 /**< \brief Adjust QP between pictures, [0,1] */
    uint32_t ctbRc;                     /**< \brief Adjust QP between Lcus, [0,1] */
    uint32_t blockRCSize;               /**< \brief Size of block rate control : 2=16x16,1= 32x32, 0=64x64*/
    int32_t qpHdr;                     /**< \brief QP for next encoded picture, [-1..51].
                                    *\n -1 = Let rate control calculate initial QP.
                                    *\n This QP is used for all pictures if
                                    * HRD and pictureRc and mbRc are disabled.
                                    * If HRD is enabled it may override this QP. */
    uint32_t qpMinPB;                   /**< \brief  Minimum QP for any P/B picture, [0..51] */
    uint32_t qpMaxPB;                   /**< \brief  Maximum QP for any P/B picture, [0..51] */
    uint32_t qpMinI;                    /**< \brief	Minimum QP for any I picture, [0..51] */
    uint32_t qpMaxI;                    /**< \brief	Maximum QP for any I picture, [0..51] */
    uint32_t bitPerSecond;              /**< \brief Target bitrate in bits/second, this is
                                    * needed if pictureRc, mbRc, pictureSkip or
                                    * hrd is enabled [10000..60000000] */
    uint32_t cpbMaxRate;                /**< \brief Cpb buffer max bitrate in bits/second, this is
                                    * needed if CPB based CBR/VBR mode */
    esclvenctrue_e fillerData;           /**< \brief enable/disable filler Data when HRD off*/
    uint32_t hrdCpbSize;                /**< \brief Size of Coded Picture Buffer in HRD (bits)
                                    * Sugest to use 2x bit rate. */
    uint32_t bitrateWindow;             /**< \brief	Length for Group of Pictures, indicates
                                    * the distance of two intra pictures,
                                    * including first intra [1..300] */
    int32_t intraQpDelta;              /**< \brief Intra QP delta. intraQP = QP + intraQpDelta
                                    * This can be used to change the relative quality
                                    * of the Intra pictures or to lower the size
                                    * of Intra pictures. [-12..12] */
    uint32_t fixedIntraQp;              /**< \brief Fixed QP value for all Intra pictures, [0..51]
                                    * 0 = Rate control calculates intra QP. */
    int32_t monitorFrames;
    uint32_t u32StaticSceneIbitPercent; /**< \brief	I frame bits percent in static scene */
    uint32_t rcQpDeltaRange;            /**< \brief  ctb rc qp delta range */
    uint32_t rcBaseMBComplexity;        /**< \brief	ctb rc mb complexity base */
    int32_t longTermQpDelta;           /**< \brief QP delta of the frame using long term reference */
    int32_t vbr;                       /**< \brief	Variable Bit Rate Control by qpMin */
    esclvencrcMode_e rcMode;             /**< \brief	RC mode (CVBR, ABR, VBR, CBR, CRF, CQP) */
    int32_t ctbRcRowQpStep;            /**< \brief	max accumulated QP adjustment step per CTB Row by Ctb Rate Control.
                                                     QP adjustment step per CTB is ctbRcRowQpStep/ctb_per_row. */
    uint32_t ctbRcQpDeltaReverse;       /**< \brief ctbrc qpdelta value reversed*/
    uint32_t frameRateNum;              /**< \brief The stream time scale, [1..1048575] */
    uint32_t frameRateDenom;            /**< \brief Maximum frame rate is frameRateNum/frameRateDenom
                                    * in frames/second. The actual frame rate will be
                                    * defined by timeIncrement of encoded pictures,
                                    * [1..frameRateNum] */
    uint32_t hrd;
    int32_t bitVarRangeP;     /**< \brief variations over average bits per frame for P frame*/
    int32_t bitVarRangeB;     /**< \brief variations over average bits per frame for B frame*/
    int32_t tolMovingBitRate; /**< \brief tolerance of max Moving bit rate. (Limitation of instantaneous bit rate) */
    int32_t picQpDeltaMin;    /**< \brief minimum pic qp delta */
    int32_t picQpDeltaMax;    /**< \brief maximum pic qp delta */
    float tolCtbRcInter;  /**< \brief Tolerance of Ctb Rate Control for INTER frames*/
    float tolCtbRcIntra;  /**< \brief Tolerance of Ctb Rate Control for INTRA frames*/
};

struct esclvencEncPictureArea
{
    uint32_t enable;		  /**< \brief [0,1] Enables this area */
    uint32_t top;		  /**< \brief Top macroblock row inside area [0..heightMbs-1] */
    uint32_t left;	      /**< \brief Left macroblock row inside area [0..widthMbs-1] */
    uint32_t bottom;		  /**< \brief Bottom macroblock row inside area [top..heightMbs-1] */
    uint32_t right; 		  /**< \brief Right macroblock row inside area [left..widthMbs-1] */
};

struct esclvencHdr10DisplaySei
{
    uint8_t hdr10_display_enable;
    uint16_t hdr10_dx0;
    uint16_t hdr10_dy0;
    uint16_t hdr10_dx1;
    uint16_t hdr10_dy1;
    uint16_t hdr10_dx2;
    uint16_t hdr10_dy2;
    uint16_t hdr10_wx;
    uint16_t hdr10_wy;
    uint32_t hdr10_maxluma;
    uint32_t hdr10_minluma;
};

struct esclvencHdr10LightLevelSei
{
    uint8_t hdr10_lightlevel_enable;
    uint16_t hdr10_maxlight;
    uint16_t hdr10_avglight;
};

struct esclvencVuiColorDescription
{
    uint8_t vuiColorDescripPresentFlag;  /**< \brief Color description present in the vui.0- not present, 1- present */
    uint8_t vuiColorPrimaries;           /**< \brief Color's Primaries */
    uint8_t vuiTransferCharacteristics;  /**< \brief Transfer Characteristics */
    uint8_t vuiMatrixCoefficients;       /**< \brief Matrix Coefficients */
};

struct esclvencVCEncCodingCtrl
{
    uint32_t sliceSize;         /**< \brief Slice size in CTB/MB rows,
                            * when the value is 0, whole frame will be encoded as one slice,
                            * otherwise (height+ctu_size-1)/ctu_size slices will be created.
                            * The height of one slice will be ctu_size*sliceSize except
                            * the last slice.*/
    uint32_t vuiVideoFullRange; /**< \brief  Input video signal sample range, [0,1]
                            * 0 = Y range in [16..235], Cb&Cr range in [16..240];
                            * 1 = Y, Cb and Cr range in [0..255] */
    /** \brief 0 = Filter enabled,
     *\n 1 = Filter disabled,
     *\n 2 = Filter disabled on slice edges. */
    uint32_t disableDeblockingFilter;
    /** \brief  Sram power down mode disable
     *\n 0 = go into power down mode when clock is gated
     *\n 1 = disable; power always on */
    int32_t tc_Offset;               /**< \brief Deblock parameter, tc_offset */
    int32_t beta_Offset;             /**< \brief Deblock parameter, beta_offset */
    uint32_t enableSao;               /**< \brief Enable SAO */
    uint32_t enableScalingList;       /**< \brief Enabled ScalingList */
    uint32_t sampleAspectRatioWidth;  /**< \brief Horizontal size of the sample aspect
                                  * ratio (in arbitrary units), 0 for unspecified, [0..65535]*/
    uint32_t sampleAspectRatioHeight; /**< \brief Vertical size of the sample aspect ratio
                                  * (in same units as sampleAspectRatioWidth)
                                  * 0 for unspecified, [0..65535]*/
    uint32_t enableCabac;             /**< \brief [0,1] H.264 entropy coding mode, 0 for CAVLC, 1 for CABAC */
    esclvencEncPictureArea intraArea;  /**< \brief Area for forcing intra macroblocks */
    esclvencEncPictureArea ipcm1Area;  /**< \brief 1st Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm2Area;  /**< \brief 2nd Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm3Area;  /**< \brief 3rd Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm4Area;  /**< \brief 4th Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm5Area;  /**< \brief 5th Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm6Area;  /**< \brief 6th Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm7Area;  /**< \brief 7th Area for forcing IPCM macroblocks */
    esclvencEncPictureArea ipcm8Area;  /**< \brief 8th Area for forcing IPCM macroblocks */
    esclvencEncPictureArea roi1Area;
    esclvencEncPictureArea roi2Area; /**< \brief Area for 2nd Region-Of-Interest */
    esclvencEncPictureArea roi3Area; /**< \brief Area for 3rd Region-Of-Interest */
    esclvencEncPictureArea roi4Area; /**< \brief Area for 4th Region-Of-Interest */
    esclvencEncPictureArea roi5Area; /**< \brief Area for 5th Region-Of-Interest */
    esclvencEncPictureArea roi6Area; /**< \brief Area for 6th Region-Of-Interest */
    esclvencEncPictureArea roi7Area; /**< \brief Area for 7th Region-Of-Interest */
    esclvencEncPictureArea roi8Area; /**< \brief Area for 8th Region-Of-Interest */
    int32_t roi1Qp;                /**< \brief [-1..51] QP value for 1st ROI, -1 means invalid. */
    int32_t roi2Qp;                /**< \brief [-1..51] QP value for 2nd ROI, -1 means invalid. */
    int32_t roi3Qp;                /**< \brief [-1..51] QP value for 3rd ROI, -1 means invalid. */
    int32_t roi4Qp;                /**< \brief [-1..51] QP value for 4th ROI, -1 means invalid. */
    int32_t roi5Qp;                /**< \brief [-1..51] QP value for 5th ROI, -1 means invalid. */
    int32_t roi6Qp;                /**< \brief [-1..51] QP value for 6th ROI, -1 means invalid. */
    int32_t roi7Qp;                /**< \brief [-1..51] QP value for 7th ROI, -1 means invalid. */
    int32_t roi8Qp;                /**< \brief [-1..51] QP value for 8th ROI, -1 means invalid. */
    int32_t chroma_qp_offset;    /**< \brief [-12...12] chroma qp offset */
    uint32_t roiMapDeltaQpEnable; /**< \brief0 = ROI map disabled, 1 = ROI map enabled. Forbidden change between frames. */
    uint32_t roiMapDeltaQpBlockUnit;    /**< \brief 0-64x64,1-32x32,2-16x16,3-8x8*/
    uint32_t ipcmMapEnable;             /**< \brief 0 = IPCM map disabled,  1 = IPCM map enabled */
    uint32_t RoimapCuCtrl_index_enable; /**< \brief 0 = ROI map cu ctrl index disabled,  1 = ROI map cu ctrl index enabled */
    uint32_t RoiQpDelta_ver;            /**< \brief QpDelta map version number */
    uint32_t skipMapEnable;             /**< \brief 0 = SKIP map disabled, 1 = SKIP map enabled */
    uint32_t firstFrameSigma;          /**< \brief valid value range :[1...30] , default :11*/
    uint32_t inputLineBufDepth;        /**< \brief input buffer depth in mb lines */
    uint32_t streamMultiSegmentAmount; /**< \brief must be >= 2*/
    esclvencHdr10DisplaySei Hdr10Display;       /**< \brief HDR10's display information in the hevc's sei */
    esclvencHdr10LightLevelSei Hdr10LightLevel; /**< \brief HDR10's light level information in the hevc's sei */
    esclvencVuiColorDescription vuiColorDescription; /**< \brief Color description in the vui, see \ref ss_vui */
    /** \brief 0..1 video signal type present in the vui flag.default 0.
     *\n 0 -- video signal type not present in	the vui
     *\n 1 -- video signal type present in the vui
     *\n See \ref ss_vui */
    uint32_t vuiVideoSignalTypePresentFlag;
    /** \brief 0..5 video format in the vui. Default 5.
     *\n 0 -- component.
     *\n 1 -- PAL.
     *\n 2 -- NTSC.
     *\n 3 -- SECAM.
     *\n 4 -- MAC.
     *\n 5 -- UNDEF.
     *\n See \ref ss_vui */
    uint32_t vuiVideoFormat;
    /** \brief 0..1 bit stream restriction present in the vui flag.default 1 as be recommended.
     *\n 0 -- bit stream restriction not present in	the vui
     *\n 1 -- bit stream restriction present in the vui
     *\n See \ref ss_vui */
    uint32_t vuiBitstreamRestrictFlag;
    uint32_t enableDeblockOverride;
    uint32_t deblockOverride;
    /** \brief Enable PCM encoding.
    * PCM encoding can be lossless. It is specified in sps. */
    int32_t pcm_enabled_flag;
    uint32_t noiseLow;  /**< \brief valid value range :[1...30] , default: 10 */
    uint32_t sbi_id_0; /**< \brief flexa sbi id 0 */
    uint32_t sbi_id_1; /**< \brief flexa sbi id 1 */
    uint32_t sbi_id_2; /**< \brief flexa sbi id 2 */
    /** \brief Dynamic Rdo enable */
    uint32_t enableDynamicRdo;
    uint32_t dynamicRdoCu16Bias;
    uint32_t dynamicRdoCu16Factor;
    uint32_t dynamicRdoCu32Bias;
    uint32_t dynamicRdoCu32Factor;
};

struct esclvencVCEncCuInfo
{
    uint8_t cuLocationX;   /**< \brief cu x coordinate relative to CTU */
    uint8_t cuLocationY;   /**< \brief cu y coordinate relative to CTU */
    uint8_t cuSize;        /**< \brief cu size. 8/16/32/64 */
    uint8_t cuMode;        /**< \brief cu mode. 0:INTER; 1:INTRA; 2:IPCM */
    uint32_t cost;          /**< \brief sse cost of cuMode*/
    uint8_t qp;
};

struct esclvenclog_env_setting
{
    esclvenc_log_output out_dir;
    esclvenc_log_level out_level;
    uint32_t k_trace_map;
    uint32_t k_check_map;
};

static const uint8_t kVencIfId = 5;
static const uint8_t kesclvencEWLInitId = 1;
static const uint8_t kesclvencEWLReleaseId = 2;
static const uint8_t kesclvencEWLAttachId = 3;
static const uint8_t kesclvencEWLDetachId = 4;
static const uint8_t kesclvencEWLGetCoreNumId = 5;
static const uint8_t kesclvencEWLMallocLinearId = 6;
static const uint8_t kesclvencEWLBindNumaIdId = 7;
static const uint8_t kesclvencVCEncInitId = 8;
static const uint8_t kesclvencVCEncReleaseId = 9;
static const uint8_t kesclvencVCEncBindNumaIdId = 10;
static const uint8_t kesclvencVCEncStrmStartId = 11;
static const uint8_t kesclvencVCEncStrmEndId = 12;
static const uint8_t kesclvencVCEncStrmEncodeId = 13;
static const uint8_t kesclvencVCEncFlushId = 14;
static const uint8_t kesclvencVCEncGetPreProcessingId = 15;
static const uint8_t kesclvencVCEncSetPreProcessingId = 16;
static const uint8_t kesclvencVCEncGetRateCtrlId = 17;
static const uint8_t kesclvencVCEncSetRateCtrlId = 18;
static const uint8_t kesclvencVCEncGetCodingCtrlId = 19;
static const uint8_t kesclvencVCEncSetCodingCtrlId = 20;
static const uint8_t kesclvencVCEncGetAlignedStrideId = 21;
static const uint8_t kesclvencVCEncGetCuInfoId = 22;
static const uint8_t kesclvencVCEncGetPerformanceId = 23;
static const uint8_t kesclvencVCEncGetAsicConfigId = 24;
static const uint8_t kesclvencVCEncGetRoiMapVersionId = 25;
static const uint8_t kesclvencVCEncSetSeiUserDataId = 26;
static const uint8_t kesclvencVCEncGetApiVersionId = 27;
static const uint8_t kesclvencVCEncGetBuildId = 28;
static const uint8_t kesclvencVCEncLogInitId = 29;
#endif // ESRPC_TYPE_DEFINITIONS_VENC

#if defined(__cplusplus)
}
#endif

#endif // _VENC_VENCIF_COMMON_H_
